package com.crypto.utils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.math.BigInteger;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.Security;
import java.security.SignatureException;
import java.security.cert.CRLException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.naming.NamingException;
import javax.naming.ldap.LdapContext;
import javax.xml.parsers.ParserConfigurationException;

import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.DERObjectIdentifier;
import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.asn1.x509.X509ObjectIdentifiers;
import org.bouncycastle.cms.CMSAlgorithm;
import org.bouncycastle.cms.CMSEnvelopedData;
import org.bouncycastle.cms.CMSEnvelopedDataGenerator;
import org.bouncycastle.cms.CMSEnvelopedDataParser;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSProcessable;
import org.bouncycastle.cms.CMSProcessableByteArray;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.CMSSignedDataGenerator;
import org.bouncycastle.cms.CMSTypedData;
import org.bouncycastle.cms.RecipientInformation;
import org.bouncycastle.cms.RecipientInformationStore;

import org.bouncycastle.cms.KeyTransRecipientInformation;
import org.bouncycastle.cms.SignerId;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;
import org.bouncycastle.cms.jcajce.JcaSignerInfoGeneratorBuilder;
import org.bouncycastle.cms.jcajce.JcaSimpleSignerInfoVerifierBuilder;
import org.bouncycastle.cms.jcajce.JceCMSContentEncryptorBuilder;
import org.bouncycastle.cms.jcajce.JceKeyTransEnvelopedRecipient;
import org.bouncycastle.cms.jcajce.JceKeyTransRecipientInfoGenerator;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.DefaultDigestAlgorithmIdentifierFinder;
import org.bouncycastle.operator.DefaultSignatureAlgorithmIdentifierFinder;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.bouncycastle.operator.jcajce.JcaDigestCalculatorProviderBuilder;
import org.xml.sax.SAXException;

import com.crypto.utils.helpers.AlgorithmIDf;
import com.crypto.utils.helpers.Base64Utils;
import com.crypto.utils.helpers.CRLHelper;

import com.crypto.utils.helpers.CerHelper;
import com.crypto.utils.helpers.KeyStoreHolder;
import com.crypto.utils.helpers.KeyStoreLoader;
import com.crypto.utils.helpers.LdapReader;
import com.crypto.utils.helpers.XmlReader;
import com.sun.crypto.provider.SunJCE;
import org.dom4j.*;
import org.dom4j.tree.AbstractText;
public class AzTrustCryptoLib {
	public static final String WINDOWS_CONTAINER_NAME = "WINDOWS";
	public static final String PKCS11_CONTAINER_NAME = "PKCS11";
	public static final String PKCS12_CONTAINER_NAME = "PFX_FILE";
	public static final String JKS_CONTAINER_NAME="JKS_FILE";
	public static String CONFIG_XML_FILE_PATH = "";

	static {

		URL location = AzTrustCryptoLib.class.getProtectionDomain()
				.getCodeSource().getLocation();

		CONFIG_XML_FILE_PATH = getXmlPath(location.getPath())
				+ "AzTrustCrypto.xml";

	}

	public String signData(byte[] inputdata, String cer_issuer,
			String cer_serial, String hash_oid, String containerName,
			String containerPassword, String containerFilePath,int detached)

	throws KeyStoreException, GeneralSecurityException, IOException,
			SecurityException, IllegalArgumentException,
			ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException,
			InvocationTargetException, OperatorCreationException, CMSException {
		KeyStoreHolder storeHolder = new KeyStoreHolder();
		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(),
					cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} 
		 else if(containerName.equals(JKS_CONTAINER_NAME)){
			 storeHolder=doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath),cer_serial,cer_issuer);
		    }
		 else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(
					storeHolder.getAlias(), passw);
			return Base64Utils.base64Encode(sign(inputdata, hash_oid, pk,
					storeHolder,detached));

		}

		return null;
	}

	public String signData(String inputdata, String cer_issuer,
			String cer_serial, String hash_oid, String containerName,
			String containerPassword, String containerFilePath,int detached)

	throws KeyStoreException, GeneralSecurityException, IOException,
			SecurityException, IllegalArgumentException,
			ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException,
			InvocationTargetException, OperatorCreationException, CMSException {
		KeyStoreHolder storeHolder = new KeyStoreHolder();

		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(),
					cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} 
		 else if(containerName.equals(JKS_CONTAINER_NAME)){
			 storeHolder=doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath),cer_serial,cer_issuer);
		    }
		else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(
					storeHolder.getAlias(), passw);
			return Base64Utils.base64Encode(sign(inputdata.getBytes(),
					hash_oid, pk, storeHolder,detached));

		}

		return null;
	}

	public String signData(String inputdata,String cer_id) throws KeyStoreException,
			GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException,
			NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException,
			OperatorCreationException, CMSException,
			ParserConfigurationException, SAXException {
		Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
		

		Map xmlCertMap = (Map) xmlMap.get(cer_id);
		String cer_issuer = (String) xmlCertMap.get("cer_issuer");
		String cer_serial = (String) xmlCertMap.get("cer_serial");
		String hash_oid = (String) xmlCertMap.get("hash_oid");
		
		String containerName = (String) xmlCertMap.get("containerName");
		String containerPassword = (String) xmlCertMap.get("containerPassword");
		String containerFilePath = (String) xmlCertMap.get("containerFilePath");
		int detached=Integer.parseInt((String) xmlCertMap.get("detached"));
		
		
		
		KeyStoreHolder storeHolder = new KeyStoreHolder();
		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(),
					cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		}
		 else if(containerName.equals(JKS_CONTAINER_NAME)){
			 storeHolder=doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath),cer_serial,cer_issuer);
		    }
		else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null || !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(
					storeHolder.getAlias(), passw);
			return Base64Utils.base64Encode(sign(inputdata.getBytes(),
					hash_oid, pk, storeHolder,detached));

		}

		return null;
	}

	public String signData(byte[] inputdata,String cer_id) throws KeyStoreException,
			GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException,
			NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException,
			OperatorCreationException, CMSException,
			ParserConfigurationException, SAXException {
		Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
		
		Map xmlCertMap = (Map) xmlMap.get(cer_id);
		String cer_issuer = (String) xmlCertMap.get("cer_issuer");
		String cer_serial = (String) xmlCertMap.get("cer_serial");
		String hash_oid = (String) xmlCertMap.get("hash_oid");
		
		String containerName = (String) xmlCertMap.get("containerName");
		String containerPassword = (String) xmlCertMap.get("containerPassword");
		String containerFilePath = (String) xmlCertMap.get("containerFilePath");
		int detached=Integer.parseInt((String) xmlCertMap.get("detached"));
		
		KeyStoreHolder storeHolder = new KeyStoreHolder();
		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(),
					cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} 
		 else if(containerName.equals(JKS_CONTAINER_NAME)){
			 storeHolder=doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath),cer_serial,cer_issuer);
		    }
		else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null || !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(
					storeHolder.getAlias(), passw);
			return Base64Utils.base64Encode(sign(inputdata, hash_oid, pk,
					storeHolder,detached));

		}

		return null;
	}

	public String verifyData(String inputdata, String CMSSignature,
			String ldapServer, String ldapPort, String ldapCertSearchDN,
			String ldp_usr, String ldp_pass) throws CMSException,
			OperatorCreationException, CertificateException, NamingException,
			ParserConfigurationException, SAXException, IOException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata
				.getBytes());
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();

		Iterator<SignerInformation> iter = signers.getSigners().iterator();

		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();

		String ldapUser = ldp_usr;
		String ldapPassword = ldp_pass;

		String Issuer = signerId.getIssuerAsString();
		Issuer=Issuer.replaceAll(",", ", ");
		String hexString = signerId.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();
		while ((sb.length() + hexString.length()) < (2 * signerId
				.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		String serialNumber = sb.toString();

		X509Certificate cert = LdapReader.getCertificate(ldapServer, ldapPort,
				ldapUser, ldapPassword, ldapCertSearchDN, serialNumber, Issuer);

		// System.out.println("signer: " + signerId.getCertificate());
		//System.out.println("signer:" + signerId.getIssuerAsString());
		boolean verified = signer
				.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider(
						"BC").build(cert));

		if (verified)
			return "verified";
		else
			return "not verified";

	}

	public String verifyData(byte[] inputdata, String CMSSignature,
			String ldapServer, String ldapPort, String ldapCertSearchDN,
			String ldp_usr, String ldp_pass) throws CMSException,
			OperatorCreationException, CertificateException, NamingException,
			ParserConfigurationException, SAXException, IOException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata
				);
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();

		Iterator<SignerInformation> iter = signers.getSigners().iterator();

		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();

		String ldapUser = ldp_usr;
		String ldapPassword = ldp_pass;

		String Issuer = signerId.getIssuerAsString();
		Issuer=Issuer.replaceAll(",", ", ");
		String hexString = signerId.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();
		while ((sb.length() + hexString.length()) < (2 * signerId
				.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		String serialNumber = sb.toString();

		X509Certificate cert = LdapReader.getCertificate(ldapServer, ldapPort,
				ldapUser, ldapPassword, ldapCertSearchDN, serialNumber, Issuer);

		// System.out.println("signer: " + signerId.getCertificate());
		//System.out.println("signer:" + signerId.getIssuerAsString());
		boolean verified = signer
				.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider(
						"BC").build(cert));

		if (verified)
			return "verified";
		else
			return "not verified";

	}

	public String verifyData(byte[] inputdata, String CMSSignature,
			String signercer) throws CMSException, OperatorCreationException,
			CertificateException, KeyStoreException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata
				);
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();

		Iterator<SignerInformation> iter = signers.getSigners().iterator();

		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();
		X509Certificate cert = getCertFromString(signercer);
		// KeyStore ks = KeyStore.getInstance("Windows-My");
		Security.addProvider(new BouncyCastleProvider());
		// Security.addProvider(ks.getProvider());
		boolean verified = signer.verify(

		new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));

		if (verified)
			return "verified";
		else
			return "not verified";

	}

	public byte[] normalize(Element elem) throws UnsupportedEncodingException {
				final StringBuilder strb = new StringBuilder();
				canonicalize(elem, strb);

				return strb.toString().getBytes("UTF-16LE");
			}

			private void canonicalize(final Element elem, final StringBuilder strb) {
				strb.append(elem.getName());

				if (elem.elements().isEmpty()) {
					boolean isEmpty = true;

					for (Iterator iter = elem.content().iterator(); iter.hasNext();) {
						Object o = iter.next();
						if (o instanceof AbstractText) {
							final String text = ((AbstractText) o).getText();
							if (!text.isEmpty()) {
								if (isEmpty) {
									isEmpty = false;
									strb.append("<=");
								}

								strb.append(text.replaceAll ("\r\n", "\n"));
							}
						}
					}

					if (isEmpty)
						strb.append("<>");
					else
						strb.append("=>");

				} else {
					strb.append("<");
					for (Object o : elem.elements()) {
						canonicalize((Element) o, strb);
					}

					strb.append(">");
				}
		}

	public String verifyData(String inputdata, String CMSSignature,
			String signercer) throws CMSException, OperatorCreationException,
			CertificateException, KeyStoreException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata
				.getBytes());
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();

		Iterator<SignerInformation> iter = signers.getSigners().iterator();

		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();
		X509Certificate cert = getCertFromString(signercer);
		// KeyStore ks = KeyStore.getInstance("Windows-My");
		Security.addProvider(new BouncyCastleProvider());
		// Security.addProvider(ks.getProvider());
		boolean verified = signer.verify(

		new JcaSimpleSignerInfoVerifierBuilder().setProvider("BC").build(cert));

		if (verified)
			return "verified";
		else
			return "not verified";

	}

	public String verifyData(String inputdata, String CMSSignature)
			throws CMSException, OperatorCreationException,
			CertificateException, NamingException,
			ParserConfigurationException, SAXException, IOException {
		Security.addProvider(new BouncyCastleProvider());
		byte[] signature = Base64Utils.base64Decode(CMSSignature);
		CMSTypedData typedData = new CMSProcessableByteArray(inputdata
				.getBytes());
		CMSSignedData s = new CMSSignedData(typedData, signature);
		SignerInformationStore signers = s.getSignerInfos();

		Iterator<SignerInformation> iter = signers.getSigners().iterator();

		SignerInformation signer = iter.next();
		SignerId signerId = signer.getSID();
		Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
		String ldapServer = (String) xmlMap.get("ldapServer");
		String ldapPort = (String) xmlMap.get("ldapPort");
		String ldapUser = (String) xmlMap.get("ldapUser");
		String ldapPassword = (String) xmlMap.get("ldapPassword");
		String ldapCertSearchDN = (String) xmlMap.get("ldapCertSearchDN");

		String Issuer = signerId.getIssuerAsString();
		Issuer=Issuer.replaceAll(",", ", ");
		String hexString = signerId.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();
		while ((sb.length() + hexString.length()) < (2 * signerId
				.getSerialNumber().toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		String serialNumber = sb.toString();

		X509Certificate cert = LdapReader.getCertificate(ldapServer, ldapPort,
				ldapUser, ldapPassword, ldapCertSearchDN, serialNumber, Issuer);

		Security.addProvider(new BouncyCastleProvider());
		boolean verified = signer
				.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider(
						"BC").build(cert));

		if (verified)
			return "verified";
		else
			return "not verified";

	}
	public String verifyData(byte[] inputdata, String CMSSignature)
	throws CMSException, OperatorCreationException,
	CertificateException, NamingException,
	ParserConfigurationException, SAXException, IOException {
Security.addProvider(new BouncyCastleProvider());
byte[] signature = Base64Utils.base64Decode(CMSSignature);
CMSTypedData typedData = new CMSProcessableByteArray(inputdata
		);
CMSSignedData s = new CMSSignedData(typedData, signature);
SignerInformationStore signers = s.getSignerInfos();

Iterator<SignerInformation> iter = signers.getSigners().iterator();

SignerInformation signer = iter.next();
SignerId signerId = signer.getSID();
Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
String ldapServer = (String) xmlMap.get("ldapServer");
String ldapPort = (String) xmlMap.get("ldapPort");
String ldapUser = (String) xmlMap.get("ldapUser");
String ldapPassword = (String) xmlMap.get("ldapPassword");
String ldapCertSearchDN = (String) xmlMap.get("ldapCertSearchDN");

String Issuer = signerId.getIssuerAsString();
Issuer=Issuer.replaceAll(",", ", ");
String hexString = signerId.getSerialNumber().toString(16);
StringBuilder sb = new StringBuilder();
while ((sb.length() + hexString.length()) < (2 * signerId
		.getSerialNumber().toByteArray().length)) {
	sb.append("0");
}
sb.append(hexString);
String serialNumber = sb.toString();

X509Certificate cert = LdapReader.getCertificate(ldapServer, ldapPort,
		ldapUser, ldapPassword, ldapCertSearchDN, serialNumber, Issuer);

Security.addProvider(new BouncyCastleProvider());
boolean verified = signer
		.verify(new JcaSimpleSignerInfoVerifierBuilder().setProvider(
				"BC").build(cert));

if (verified)
	return "verified";
else
	return "not verified";

}


	public String encryptData(String inputdata, String recipientCert)
			throws IOException, CertificateException, KeyStoreException {
		byte[] inputd = inputdata.getBytes();
		X509Certificate recipientCer = getCertFromString(recipientCert);
		CMSEnvelopedDataGenerator edGen = new CMSEnvelopedDataGenerator();

		CMSEnvelopedData ed;

		Security.addProvider(new BouncyCastleProvider());
		

		try {
			Security.insertProviderAt(new SunJCE(),1);
			edGen
					.addRecipientInfoGenerator(new JceKeyTransRecipientInfoGenerator(
							recipientCer).setProvider("BC"));
			ed = edGen
					.generate(new CMSProcessableByteArray(inputd),
							new JceCMSContentEncryptorBuilder(
									CMSAlgorithm.DES_EDE3_CBC)
									.setProvider("BC").build());
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}

		return Base64Utils.base64Encode(ed.getEncoded());
	}

	private String decrypteData(String inputdata) throws KeyStoreException,
			GeneralSecurityException, IOException, SecurityException,
			IllegalArgumentException, ClassNotFoundException,
			NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException,
			OperatorCreationException, CMSException,
			ParserConfigurationException, SAXException {
		Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
		String cer_issuer = (String) xmlMap.get("cer_issuer");
		String cer_serial = (String) xmlMap.get("cer_serial");
		String hash_oid = (String) xmlMap.get("hash_oid");
		String containerName = (String) xmlMap.get("containerName");
		String containerPassword = (String) xmlMap.get("containerPassword");
		String containerFilePath = (String) xmlMap.get("containerFilePath");
		KeyStoreHolder storeHolder = new KeyStoreHolder();
		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(),
					cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} 
		else if(containerName.equals(JKS_CONTAINER_NAME)){
			 storeHolder=doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath),cer_serial,cer_issuer);
		    }
		else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null || !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(
					storeHolder.getAlias(), passw);
			return Base64Utils
					.base64Encode(decrypt(inputdata, pk, storeHolder));

		}

		return null;
	}

	public  String decrypteData(String inputdata, String cer_issuer,
			String cer_serial, String hash_oid, String containerName,
			String containerPassword, String containerFilePath)

	throws KeyStoreException, GeneralSecurityException, IOException,
			SecurityException, IllegalArgumentException,
			ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException,
			InvocationTargetException, OperatorCreationException, CMSException {
		KeyStoreHolder storeHolder = new KeyStoreHolder();

		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromWindows(),
					cer_serial, cer_issuer);
		} else if (containerName.equals(PKCS11_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS11(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			storeHolder = doFilter(KeyStoreLoader.loadStoreFromPKCS12(
					containerPassword, containerFilePath), cer_serial,
					cer_issuer);
		} 
		else if(containerName.equals(JKS_CONTAINER_NAME)){
			 storeHolder=doFilter(KeyStoreLoader.loadStoreFromJKS(containerPassword, containerFilePath),cer_serial,cer_issuer);
		    }

		else {
			try {
				throw new Throwable("Container has not been declared");
			} catch (Throwable e) {
			}
		}
		if (storeHolder != null) {
			char[] passw = null;
			if (containerPassword != null && !containerPassword.equals(""))
				passw = containerPassword.toCharArray();

			PrivateKey pk = (PrivateKey) storeHolder.getStore().getKey(
					storeHolder.getAlias(), passw);
			return new String(decrypt(inputdata, pk, storeHolder),"UTF-8");

		}

		return null;
	}

	private byte[] decrypt(String encodedCmss, PrivateKey key,
			KeyStoreHolder sHolder) throws CMSException, IOException,
			NoSuchProviderException, KeyStoreException {
		byte[] encodedCms = Base64Utils.base64Decode(encodedCmss);
		Security.addProvider(new com.sun.crypto.provider.SunJCE());
		
		
		//Security.addProvider(KeyStore.getInstance("Windows-my").getProvider());
	
		CMSEnvelopedDataParser envelopedDataParser = new CMSEnvelopedDataParser(
				new ByteArrayInputStream(encodedCms));
		
		CMSEnvelopedData enveloped = new CMSEnvelopedData(encodedCms);
		
		RecipientInformationStore recipients = enveloped.getRecipientInfos();
		Collection recc = recipients.getRecipients();
	    RecipientInformation res=(RecipientInformation) recc.iterator().next();
		
		Security.addProvider(sHolder.getStore().getProvider());
		//System.out.println(sHolder.getStore().getProvider());
		Security.addProvider(new BouncyCastleProvider());
		byte[] content=null;
		if(sHolder.getStore().getProvider().getName().startsWith("SunPKCS11")||
				sHolder.getStore().getProvider().getName().startsWith("SunMSCAPI")		)
			content= res.getContent(key,sHolder.getStore().getProvider());
		else 
			content= res.getContent(key,"BC");
		return content;

	}
	
	
/*	private byte[] decrypt(String encodedCmss, PrivateKey key,
			KeyStoreHolder sHolder) throws CMSException, IOException,
			NoSuchProviderException {
		byte[] encodedCms = Base64Utils.base64Decode(encodedCmss);
		Security.addProvider(new com.sun.crypto.provider.SunJCE());
		Security.addProvider(new BouncyCastleProvider());
		Security.addProvider(sHolder.getStore().getProvider());
		byte[] contents = null;
		CMSEnvelopedDataParser envelopedDataParser = new CMSEnvelopedDataParser(
				new ByteArrayInputStream(encodedCms));
		
		CMSEnvelopedData enveloped = new CMSEnvelopedData(encodedCms);
		Collection recip = enveloped.getRecipientInfos().getRecipients();
	    RecipientInformation   recipient = (RecipientInformation)recip.iterator().next();
	    
	    contents=recipient.getContent(new JceKeyTransEnvelopedRecipient(key).setProvider("BC"));
		KeyTransRecipientInformation rinfo = (KeyTransRecipientInformation) recip
				.iterator().next();
System.out.println(rinfo.getKeyEncryptionAlgOID());
		contents = rinfo.getContent(new JceKeyTransEnvelopedRecipient(key)
				.setProvider("BC"));

		return contents;

	}*/

	public String getCMSSerNum(String cmsSignature) throws CMSException,
			NoSuchAlgorithmException, NoSuchProviderException {
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSSignedData signedData = new CMSSignedData(signature);
		SignerInformationStore signers = signedData.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		return getSerial(signer.getSID().getSerialNumber());

	}

	public String getCMSAlgorithmName(String cmsSignature) throws CMSException,
			NoSuchAlgorithmException, NoSuchProviderException {
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSSignedData signedData = new CMSSignedData(signature);
		SignerInformationStore signers = signedData.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		return AlgorithmIDf.getSignatureAlgorithmName(signer.toASN1Structure()
				.getDigestAlgorithm(), signer.toASN1Structure()
				.getDigestEncryptionAlgorithm());

	}

	public String getCMSAlgorithmOID(String cmsSignature) throws CMSException,
			NoSuchAlgorithmException, NoSuchProviderException {
		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSSignedData signedData = new CMSSignedData(signature);
		SignerInformationStore signers = signedData.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		AlgorithmIdentifier sigAlg = new DefaultSignatureAlgorithmIdentifierFinder()
				.find(getCMSAlgorithmName(cmsSignature));
		return sigAlg.getAlgorithm().getId();

	}

	public String getCMSIssuer(String cmsSignature) throws CMSException,
			NoSuchAlgorithmException, NoSuchProviderException {

		byte[] signature = Base64Utils.base64Decode(cmsSignature);
		CMSSignedData signedData = new CMSSignedData(signature);
		SignerInformationStore signers = signedData.getSignerInfos();
		Iterator<SignerInformation> iter = signers.getSigners().iterator();
		SignerInformation signer = iter.next();
		String Issuer= signer.getSID().getIssuerAsString();
		Issuer=Issuer.replaceAll(",", ", ");
		return Issuer;

	}

	public String getCertificate(String ldapServer, String ldapPort,
			String ldapCertSearchDN, String ldp_usr, String ldp_pass,
			String cer_issuer, String cer_serial) throws CertificateException,
			NamingException {

		return Base64Utils.base64Encode(LdapReader.getCertificate(ldapServer,
				ldapPort, ldp_usr, ldp_pass, ldapCertSearchDN, cer_serial,
				cer_issuer).getEncoded());

	}

	public String getCertificate(String cer_issuer, String cer_serial,String cer_id)
			throws CertificateEncodingException, CertificateException,
			NamingException, ParserConfigurationException, SAXException,
			IOException {
		Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
		String ldapServer = (String) xmlMap.get("ldapServer");
		String ldapPort = (String) xmlMap.get("ldapPort");
		String ldp_usr = (String) xmlMap.get("ldapUser");
		String ldp_pass = (String) xmlMap.get("ldapPassword");
		String ldapCertSearchDN = (String) xmlMap.get("ldapCertSearchDN");
		return Base64Utils.base64Encode(LdapReader.getCertificate(ldapServer,
				ldapPort, ldp_usr, ldp_pass, ldapCertSearchDN, cer_serial,
				cer_issuer).getEncoded());

	}

	public String getCertificate(String cer_url) throws FileNotFoundException, CertificateException, CRLException, IOException{
		
		X509Certificate c509=null;
		String x509=null;
		String beginCert="-----BEGIN CERTIFICATE-----";
		String endCert = "-----END CERTIFICATE-----";
		if (cer_url.startsWith("http://") || cer_url.startsWith("https://")
				 
                || cer_url.startsWith("ftp://")) {

             c509=CerHelper.downloadCerFromWeb(cer_url);
		}
            else  
            	c509=CerHelper.readCertFromFile(cer_url);
          x509=beginCert+Base64Utils.base64Encode(c509.getEncoded())+endCert;
            return  x509;
       }
		
	
	
	
	
	
	public String getCertificate(String cer_serial,String cer_id)
			throws CertificateEncodingException, CertificateException,
			NamingException, ParserConfigurationException, SAXException,
			IOException {
		Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
	
		String ldapServer = (String) xmlMap.get("ldapServer");
		String ldapPort = (String) xmlMap.get("ldapPort");
		String ldp_usr = (String) xmlMap.get("ldapUser");
		String ldp_pass = (String) xmlMap.get("ldapPassword");
		String ldapCertSearchDN = (String) xmlMap.get("ldapCertSearchDN");
		Map xmlCertMap = (Map) xmlMap.get(cer_id);
		String cer_issuer = (String) xmlCertMap.get("cer_issuer");
		return Base64Utils.base64Encode(LdapReader.getCertificate(ldapServer,
				ldapPort, ldp_usr, ldp_pass, ldapCertSearchDN, cer_serial,
				cer_issuer).getEncoded());

	}

	public String ConvBlock4(String block4) throws Exception {

		String stringToSign = block4.replaceAll("\r\n", "\n");
		return new String(stringToSign.getBytes("UTF-16LE"));

	}
	public byte[] ConvBlock4(byte[] block4) throws Exception  {

		String stringToSign = new String(block4).replaceAll("\r\n", "\n");
		return stringToSign.getBytes("UTF-16LE");

	}

	public boolean checkCerValidity(String certificate, Date val_date)
			throws CertificateException {

		X509Certificate x509 = getCertFromString(certificate);
		try {
			x509.checkValidity(val_date);
			return true;
		} catch (CertificateExpiredException ex) {
			return false;
		} catch (CertificateNotYetValidException ex) {
			return false;
		}
	}

	public boolean checkCerAuthority(String endcer, String authcer)
			throws CertificateException, InvalidKeyException,
			NoSuchAlgorithmException, NoSuchProviderException {
		X509Certificate x509endcer = getCertFromString(endcer);
		X509Certificate x509authcer = getCertFromString(authcer);
		try {
			x509endcer.verify(x509authcer.getPublicKey());
			return true;
		} catch (SignatureException ex) {
			return false;
		}

	}
	public  boolean verifyCRLFromLDAP(String certificate, String ldapServer,String ldapPort, String ldapCertSearchDN, String ldp_usr, String ldp_pass, String cer_issuer) throws CertificateException, CRLException, NamingException
	{

		X509Certificate cert=getCertFromString(certificate);
		X509CRL crl=CRLHelper.downloadCRLFromLDAP(ldapServer, ldapPort, ldp_usr, ldp_pass, ldapCertSearchDN,cer_issuer);
		if(!crl.isRevoked(cert))
		{ return true;}
		return 	false;	
	}
	public boolean verifyCRLFromWeb(String certificate,String URL) throws MalformedURLException, CertificateException, CRLException, IOException
	{
		X509Certificate cert=getCertFromString(certificate);
		X509CRL crl=CRLHelper.readCRLFromWeb(cert,URL);
		if(!crl.isRevoked(cert))
		{ return true;}
		return 	false;	
	}
	public boolean verifyCRLFromFile(String certificate, String pathToFile) throws FileNotFoundException, CertificateException, CRLException, IOException
	{
		X509Certificate cert=getCertFromString(certificate);
		
		X509CRL crl=CRLHelper.readCRLFromFile(pathToFile);
		if(!crl.isRevoked(cert))
		{ return true;}
		return 	false;
	}
	private byte[] sign(byte[] data, String algorithmID, PrivateKey privateKey,
			KeyStoreHolder storeHolder,int detached) throws OperatorCreationException,
			CertificateEncodingException, NoSuchAlgorithmException,
			NoSuchProviderException, CMSException, IOException {
		CMSTypedData typedData = new CMSProcessableByteArray(data);

		CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
        
		Security.addProvider(new BouncyCastleProvider());
		ContentSigner signer=null;
		//for jks default Sun provider  
	    if(storeHolder.getStore().getProvider().getName().equals("SUN"))
		 signer = new JcaContentSignerBuilder(
				(String) AlgorithmIDf.getOids().get(algorithmID)).setProvider("BC").build(privateKey);
	    // for pfx  
	    else if(storeHolder.getStore().getProvider().getName().equals("SunJSSE"))
	    	 signer = new JcaContentSignerBuilder(
	 				(String) AlgorithmIDf.getOids().get(algorithmID)).setProvider("SunRsaSign").build(privateKey);
	 	 //mscapi + pkcs11 ..  
	    else
	     signer = new JcaContentSignerBuilder(
						(String) AlgorithmIDf.getOids().get(algorithmID)).setProvider(storeHolder.getStore().getProvider()).build(privateKey);
			
		
		
		JcaSignerInfoGeneratorBuilder builder = new JcaSignerInfoGeneratorBuilder(
				new JcaDigestCalculatorProviderBuilder().setProvider("BC")
						.build());

		gen.addSignerInfoGenerator(builder.build(signer, storeHolder
				.getCertificate()));
      boolean isdetached=false;
      if(detached==1){isdetached=true;}
		CMSSignedData signed = gen.generate(typedData, isdetached, "BC");
		signed = new CMSSignedData(signed.getEncoded());
       
		byte[] der = signed.getEncoded();

		return der;
	}


	private KeyStoreHolder doFilter(List list, String serialNum, String issuerCN) {

		String serial = serialNum.replaceAll(" ", "");
		String issuer = issuerCN;

		for (int a = 0; a < list.size(); a++) {
			KeyStoreHolder model = (KeyStoreHolder) list.get(a);

			

			String iss = model.getCertificate().getIssuerDN().getName();
			BigInteger ser = new BigInteger(serial, 16);
			
			if (ser.equals(model.getCertificate().getSerialNumber()) && iss.equals(issuer))

			{

				return model;
			}
		}
		return null;
	}

	private X509Certificate getCertFromString(String cert)
			throws CertificateException {
		String begin = "-----BEGIN CERTIFICATE-----";
		String end = "-----END CERTIFICATE-----";
		String str = "";
		cert = cert.replaceAll("(\r\n|\n)", "");
		if (cert.startsWith(begin) && cert.endsWith(end)) {
			cert = cert.replace(begin, "");
			cert = cert.replace(end, "");

		}
		
		
		CertificateFactory certFactory = CertificateFactory
				.getInstance("X.509");
		InputStream in = new ByteArrayInputStream(Base64Utils
				.base64Decode(cert));
		X509Certificate x509cert = (X509Certificate) certFactory
				.generateCertificate(in);
		return x509cert;
	}

	private X509Certificate getCertFromCerFile(String file_path)
			throws CertificateException, IOException {

		CertificateFactory certFactory = CertificateFactory
				.getInstance("X.509");
		InputStream in = new ByteArrayInputStream(XmlReader
				.fileToByteArray(file_path));
		X509Certificate x509cert = (X509Certificate) certFactory
				.generateCertificate(in);
		return x509cert;
	}

	private static String getXmlPath(String line) {
		String newLine = "";
		String[] z = line.split("/");
		for (int i = 1; i < z.length - 1; i++) {
			newLine += z[i] + "//";
		}
		newLine = newLine.replaceAll("%20", " ");
		return newLine;
	}

	private Provider getProvider(String name)
			throws ParserConfigurationException, SAXException, IOException,
			KeyStoreException {
		String containerName = "";
		KeyStore ks = null;
		if (name == null) {
			Map xmlMap = XmlReader.readXmlFile(CONFIG_XML_FILE_PATH);
			containerName = (String) xmlMap.get("containerName");
		} else {
			containerName = name;
		}

		if (containerName.equals(WINDOWS_CONTAINER_NAME)) {
			ks = KeyStore.getInstance("Windows-My");
		}

		else if (containerName.equals(PKCS11_CONTAINER_NAME)) {

			ks = KeyStore.getInstance("PKCS11");
		} else if (containerName.equals(PKCS12_CONTAINER_NAME)) {
			ks = KeyStore.getInstance("PKCS12");
		}

		return ks.getProvider();

	}

	private static String getSerial(BigInteger SerialNumber) {
		String hexString = SerialNumber.toString(16);
		StringBuilder sb = new StringBuilder();

		while ((sb.length() + hexString.length()) < (2 * SerialNumber
				.toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		return sb.toString();
	}

	private String getSerial(X509Certificate cert) {
		String hexString = cert.getSerialNumber().toString(16);
		StringBuilder sb = new StringBuilder();

		while ((sb.length() + hexString.length()) < (2 * cert.getSerialNumber()
				.toByteArray().length)) {
			sb.append("0");
		}
		sb.append(hexString);
		return sb.toString();
	}
}
