package com.crypto.utils.helpers;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.cert.CRLException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.CertificateParsingException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

import org.bouncycastle.asn1.ASN1InputStream;
import org.bouncycastle.asn1.DERIA5String;
import org.bouncycastle.asn1.DERObject;
import org.bouncycastle.asn1.DEROctetString;
import org.bouncycastle.asn1.x509.CRLDistPoint;
import org.bouncycastle.asn1.x509.DistributionPoint;
import org.bouncycastle.asn1.x509.DistributionPointName;
import org.bouncycastle.asn1.x509.GeneralName;
import org.bouncycastle.asn1.x509.GeneralNames;
import org.bouncycastle.asn1.x509.X509Extensions;

public class CRLHelper {
	public static X509CRL readCRLFromWeb (X509Certificate cert,String crlURL) throws MalformedURLException, CertificateException, CRLException, IOException{
		
		/*    List<String> crlDistPoints =getCrlDistributionPoints( cert);
		  
		   if(!crlDistPoints.isEmpty()){ 
           
           for (String crlURL : crlDistPoints) {*/
           	
               
		  
		   if (crlURL.startsWith("http://") || crlURL.startsWith("https://")
					 
	                 || crlURL.startsWith("ftp://")) {
	 
	             X509CRL crl = downloadCRLFromWeb(crlURL);
	             
	             return crl;
	        } 

		   
		   
           
	
		   return null;
	}
	  private static LdapContext  login(String ldapServer,String ldapPort,String ldapUser,String ldapPassword) throws NamingException {
	        
          Hashtable authEnv = new Hashtable();
          String ldapURL = "ldap://"+ldapServer+":"+ldapPort;
          authEnv.put(Context.INITIAL_CONTEXT_FACTORY,"com.sun.jndi.ldap.LdapCtxFactory");
          authEnv.put(Context.PROVIDER_URL, ldapURL);
          authEnv.put(Context.SECURITY_AUTHENTICATION, "simple");
          //authEnv.put(Context.SECURITY_PRINCIPAL, ldapUser);
         // authEnv.put(Context.SECURITY_CREDENTIALS, ldapPassword);
           
          return  new InitialLdapContext(authEnv, null);  
      
      }

private static  X509CRL   search(String Issuer,LdapContext ctx,String dir) throws NamingException,
                                                                                      CertificateException, CRLException {
        SearchControls sc = new SearchControls();
        String[] attributeFilter = {"cn","certificateRevocationList"};
        sc.setReturningAttributes(attributeFilter);
        sc.setSearchScope(SearchControls.SUBTREE_SCOPE);

  String filter = "(cn="+Issuer+")";


        NamingEnumeration results = ctx.search(dir, filter, sc);
        SearchResult sr = (SearchResult) results.next();
        
        Attributes attrs = sr.getAttributes();
        
        System.out.println();
        Attribute aval = attrs.get("certificateRevocationList");
        
        byte[] val = (byte[])aval.get();

        if ((val == null) || (val.length == 0)) {
    
        ctx.close();
       
       	 return null;

        } else {

            InputStream inStream = new ByteArrayInputStream(val);

            CertificateFactory cf = CertificateFactory.getInstance("X.509");
           
            X509CRL crl = (X509CRL)cf.generateCRL(inStream);
            ctx.close();
            System.out.println(crl.getIssuerDN());
            return crl;

           
        }
      
        
           
        }
	public static  X509Certificate readCertFromFile(String filePath) throws FileNotFoundException, IOException, CertificateException, CRLException
	{
		
		 InputStream inStream = new ByteArrayInputStream(XmlReader.fileToByteArray(filePath));
		 CertificateFactory cf = CertificateFactory.getInstance("X.509");
		 X509Certificate cert = (X509Certificate)cf.generateCertificate(inStream);
         return cert;
		
	}
	public static  X509CRL readCRLFromFile(String filePath) throws FileNotFoundException, IOException, CertificateException, CRLException
	{
		
		 InputStream inStream = new ByteArrayInputStream(XmlReader.fileToByteArray(filePath));
		 CertificateFactory cf = CertificateFactory.getInstance("X.509");
         X509CRL crl = (X509CRL)cf.generateCRL(inStream);
         return crl;
		
	}
	public static X509CRL downloadCRLFromLDAP(String ldapServer,String ldapPort,String ldapUser,String ldapPassword
            ,String ldapCertSearchDN,String Issuer) throws NamingException, CertificateException, CRLException{
	LdapContext ctx=login(ldapServer, ldapPort, ldapUser, ldapPassword);	
    return search( Issuer, ctx, ldapCertSearchDN);
     }
 
	public static X509CRL downloadCRLFromWeb(String crlURL)
	  
      throws MalformedURLException, IOException, CertificateException,

      CRLException {

  URL url = new URL(crlURL);

  InputStream crlStream = url.openStream();

  try {

      CertificateFactory cf = CertificateFactory.getInstance("X.509");

      X509CRL crl = (X509CRL) cf.generateCRL(crlStream);

      return crl;

  } finally {

      crlStream.close();

  }

}
	
private static List<String> getCrlDistributionPoints(X509Certificate cert) throws CertificateParsingException, IOException {
		    
		  byte[] crldpExt = cert.getExtensionValue(X509Extensions.CRLDistributionPoints.getId());

		   
          if (crldpExt == null) {

        List<String>emptyList = new ArrayList<String>();

        return emptyList;

    }

    ASN1InputStream oAsnInStream = new ASN1InputStream(

            new ByteArrayInputStream(crldpExt));

    DERObject derObjCrlDP = oAsnInStream.readObject();

    DEROctetString dosCrlDP = (DEROctetString) derObjCrlDP;

    byte[] crldpExtOctets = dosCrlDP.getOctets();

    ASN1InputStream oAsnInStream2 = new ASN1InputStream(

            new ByteArrayInputStream(crldpExtOctets));

    DERObject derObj2 = oAsnInStream2.readObject();

    CRLDistPoint distPoint = CRLDistPoint.getInstance(derObj2);

    List<String> crlUrls = new ArrayList<String>();

    for (DistributionPoint dp : distPoint.getDistributionPoints()) {

        DistributionPointName dpn = dp.getDistributionPoint();
        // Look for URIs in fullName

        if (dpn != null) {

            if (dpn.getType() == DistributionPointName.FULL_NAME) {
                GeneralName[] genNames = GeneralNames.getInstance(

                    dpn.getName()).getNames();

                // Look for an URI
                for (int j = 0; j < genNames.length; j++) {

                    if (genNames[j].getTagNo() == GeneralName.uniformResourceIdentifier) {

                        String url = DERIA5String.getInstance(

                            genNames[j].getName()).getString();
                       
                        crlUrls.add(url);

                    }

                }

            }

        }

    }

    return crlUrls;

}
}
