package com.crypto.utils.helpers;
import java.lang.reflect.Field;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.security.GeneralSecurityException;
import java.security.Key;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.KeyStoreSpi;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.Security;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import javax.security.auth.callback.CallbackHandler;

public class KeyStoreLoader {
	private static KeyStore currentStore;
	ArrayList<String> aliasLine = new ArrayList<String>();

	public static KeyStore getCurrentStore() {
		return currentStore;
	}

	private static void setCurrentStore(KeyStore currentStore) {
		KeyStoreLoader.currentStore = currentStore;
	}

	public static ArrayList<KeyStoreHolder> loadStoreFromWindows()
			throws KeyStoreException, GeneralSecurityException, IOException {
		ArrayList<X509Certificate> certList = new ArrayList<X509Certificate>();
		
		ArrayList<KeyStoreHolder> models = new ArrayList<KeyStoreHolder>();
		String storeType = "WINDOWS-my";
		KeyStore store = loadKeyStore(null, null, storeType);
		 _fixAliases(store);
		Enumeration enumAl = store.aliases();

		while (enumAl.hasMoreElements()) {
			KeyStoreHolder model = new KeyStoreHolder();
			String alName = (String) enumAl.nextElement();
			java.security.cert.Certificate cert = store.getCertificate(alName);
			X509Certificate xCert = (X509Certificate) cert;

			model.setAlias(alName);
			model.setCertificate(xCert);
			//model.setPrivateKey((PrivateKey)store.getKey(alName,null));
			model.setStore(store);
			models.add(model);
		}

		setCurrentStore(store);
		return models;

	}
	public static ArrayList<KeyStoreHolder> loadStoreFromJKS(String password,
			String jksFilePath) throws KeyStoreException,
			NoSuchAlgorithmException, CertificateException,
			FileNotFoundException, IOException, UnrecoverableKeyException {
		ArrayList<X509Certificate> certList = new ArrayList<X509Certificate>();
		KeyStore keyStore = KeyStore.getInstance("JKS");
		keyStore.load(new FileInputStream(new File(jksFilePath)), password
				.toCharArray());
		  
		Enumeration e = keyStore.aliases();
       
		
		ArrayList<KeyStoreHolder> models = new ArrayList<KeyStoreHolder>();

		while (e.hasMoreElements()) {
			KeyStoreHolder model = new KeyStoreHolder();
			String alias = (String) e.nextElement();
			X509Certificate c = (X509Certificate) keyStore
					.getCertificate(alias);
			// certList.add(c);

			model.setAlias(alias);
			model.setCertificate(c);
			model.setStore(keyStore);
			//model.setPrivateKey((PrivateKey)keyStore.getKey(alias,password.toCharArray()));
			models.add(model);
		}
		setCurrentStore(keyStore);
		return models;
	}

	
	public static ArrayList<KeyStoreHolder> loadStoreFromPKCS12(String password,
			String pfxFilePath) throws KeyStoreException,
			NoSuchAlgorithmException, CertificateException,
			FileNotFoundException, IOException, UnrecoverableKeyException {
		ArrayList<X509Certificate> certList = new ArrayList<X509Certificate>();
		KeyStore keyStore = KeyStore.getInstance("PKCS12");
		keyStore.load(new FileInputStream(new File(pfxFilePath)), password
				.toCharArray());
		  
		Enumeration e = keyStore.aliases();
       
		
		ArrayList<KeyStoreHolder> models = new ArrayList<KeyStoreHolder>();

		while (e.hasMoreElements()) {
			KeyStoreHolder model = new KeyStoreHolder();
			String alias = (String) e.nextElement();
			X509Certificate c = (X509Certificate) keyStore
					.getCertificate(alias);
			// certList.add(c);

			model.setAlias(alias);
			model.setCertificate(c);
			model.setStore(keyStore);
			//model.setPrivateKey((PrivateKey)keyStore.getKey(alias,password.toCharArray()));
			models.add(model);
		}
		setCurrentStore(keyStore);
		return models;
	}

	public static ArrayList<KeyStoreHolder> loadStoreFromPKCS11(String pin,
			String dllFilePath) throws KeyStoreException,
			ClassNotFoundException, SecurityException, NoSuchMethodException,
			IllegalArgumentException, InstantiationException,
			IllegalAccessException, InvocationTargetException,
			NoSuchAlgorithmException, CertificateException, IOException, UnrecoverableKeyException {
		ArrayList<X509Certificate> certList = new ArrayList<X509Certificate>();
		String pkcs11Config = "name = SmartCard"
				+ System.getProperty("line.separator") + "library = "
				+ dllFilePath;
		byte[] pkcs11configBytes = pkcs11Config.getBytes();
		ByteArrayInputStream configStream = new ByteArrayInputStream(
				pkcs11configBytes);
		/*
		 * Provider pkcs11Provider = new
		 * sun.security.pkcs11.SunPKCS11(configStream);
		 * Security.addProvider(pkcs11Provider); CallbackHandler cmdLineHdlr =
		 * new com.sun.security.auth.callback.DialogCallbackHandler();
		 * KeyStore.Builder builder = KeyStore.Builder.newInstance("PKCS11",
		 * pkcs11Provider, new KeyStore.CallbackHandlerProtection(cmdLineHdlr));
		 * KeyStore keyStore = builder.getKeyStore();
		 */
		Class sunPkcs11Class = Class.forName("sun.security.pkcs11.SunPKCS11");
		Constructor pkcs11Constr = sunPkcs11Class
				.getConstructor(java.io.InputStream.class);
		Provider pkcs11Provider = (Provider) pkcs11Constr
				.newInstance(configStream);
		Security.addProvider(pkcs11Provider);
		KeyStore keyStore = KeyStore.getInstance("PKCS11");
		keyStore.load(null, pin.toCharArray());
		
		Enumeration e = keyStore.aliases();
		
		ArrayList<KeyStoreHolder> models = new ArrayList<KeyStoreHolder>();
		while (e.hasMoreElements()) {
			KeyStoreHolder model = new KeyStoreHolder();
			String alias = (String) e.nextElement();
			X509Certificate c = (X509Certificate) keyStore
					.getCertificate(alias);
			// certList.add(c);

			model.setAlias(alias);
			model.setCertificate(c);
			model.setStore(keyStore);
			//model.setPrivateKey((PrivateKey)keyStore.getKey(alias,pin.toCharArray()));

			models.add(model);
		}
		setCurrentStore(keyStore);
		return models;

	}

	private static ArrayList<X509Certificate> loadStoreFromPKCS7(
			String p7bFilePath) throws FileNotFoundException,
			CertificateException {
		ArrayList<X509Certificate> certList = new ArrayList<X509Certificate>();
		FileInputStream is;

		is = new FileInputStream(new File(p7bFilePath));
		CertificateFactory cf = CertificateFactory.getInstance("X.509");

		Iterator i = cf.generateCertificates(is).iterator();

		while (i.hasNext()) {
			X509Certificate c = (X509Certificate) i.next();
			certList.add(c);
			// TODO encode c as Base64...
		}

		return certList;
	}
    private static void _fixAliases(KeyStore keyStore) {
        Field field;
        KeyStoreSpi keyStoreVeritable;

        try {
            field = keyStore.getClass().getDeclaredField("keyStoreSpi");
            field.setAccessible(true);
            keyStoreVeritable = (KeyStoreSpi) field.get(keyStore);

            if ("sun.security.mscapi.KeyStore$MY".equals(keyStoreVeritable.getClass().getName())) {
                Collection entries;
                String alias, hashCode;
                X509Certificate[] certificates;

                field = keyStoreVeritable.getClass().getEnclosingClass().getDeclaredField("entries");
                field.setAccessible(true);
                entries = (Collection) field.get(keyStoreVeritable);

                for (Object entry : entries) {
                    field = entry.getClass().getDeclaredField("certChain");
                    field.setAccessible(true);
                    certificates = (X509Certificate[]) field.get(entry);

                    hashCode = Integer.toString(certificates[0].hashCode());

                    field = entry.getClass().getDeclaredField("alias");
                    field.setAccessible(true);
                    alias = (String) field.get(entry);

                    if (!alias.equals(hashCode)) {
                        field.set(entry, alias.concat(" - ").concat(hashCode));
                    } // if
                } // for
            } // if
        } catch (Exception exception) {
            System.err.println(exception);
            exception.printStackTrace();
        } // catch
    } // _fixAliases

	private static KeyStore loadKeyStore(String aFileName,
			String aKeyStorePasswd, String StoreType)
			throws GeneralSecurityException, IOException {
		FileInputStream keyStoreStream = null;
		char[] password = null;
		KeyStore keyStore = KeyStore.getInstance(StoreType);
		if (aFileName != null) {
			keyStoreStream = new FileInputStream(aFileName);
		}
		if (aKeyStorePasswd != null) {
			password = aKeyStorePasswd.toCharArray();
		}

		keyStore.load(keyStoreStream, password);
		return keyStore;
	}
}
